@@ -13,11 +13,42 @@
 # 0.1	2010-03-09	First steps, based on a gstreamer+python example
 #			added a treeview widget with code to scan and
 #			display /home/usr/MyDocs/.sounds
+#
 # 0.2	2010-03-22	added code to load/save/display prefs, added timeout
 #			bug fixing
+#
 # 0.3	2010-03-25	fixed some bugs (timeout never did not work)			
 #			added slider
 #
+# 0.4	2010-04-11	display Metatags as an alternate to Filenames while playing (Progressbar)
+#			rework of preferences and file scanning code
+#			added .flac type
+#			now recognices all types .mp3/.MP3/.Mp3 etc
+#			Bug reported and fixed by Frank Boer (Thanks!)
+#			- pause eating cpu cycles
+#			- no sound output in silent mode
+#
+# TODO:
+#
+# * save position in song/audio book after shutdown/sleep
+# * remove STOP button. Instead use Play/Pause. Pause will completly stop
+#   playback but remembers position in song and resumes to this. Currently
+#   Pause sets gstream to wait mode
+#
+# * rearange buttons in portrait mode
+# * do a backround rescan every startup (seperate thread) and then update playlist
+# * option to manually delete files on the device (long press in directory browser?)
+#   delete complete directories?
+# * show file which is playing in the browser  
+# * add volume bar / button which shows volume bar and automatically hides it
+# * delete files from playlist which don't play
+# * Support for sleeper and headphoned over D-Bus. See
+#   http://talk.maemo.org/showthread.php?t=48414&highlight=easyplayer
+#
+# * KISS Mode (just buttons: prev / play_stop / next and progressbar), big buttons
+# * use controls via bluetooth
+# * more sublevels in browser
+# 
 ##########################################################################
 
 import sys, os, thread, time, string, pickle
@@ -28,7 +59,7 @@ import gst
 import hildon
 from portrait import FremantleRotation
 
-version = "0.3 - 2010-03-29"
+version = "0.4 - 2010-04-11"
 # Enable debugging
 dbg = 0
 
@@ -40,13 +71,15 @@ class GTK_Main:
     self.pl_files = [] 	# list to store filenames of music files
     self.max_files = 0	# number of entries stored in playlist	
     
-    self.play_name = ""		#  Name of actual playing audio file
+    self.play_name = ""		# Name of actual playing audio file
     self.act_file_id = 0	# id of above
     self.act_file_len = 0	# len in nano-seconds of above
 
     self.pause_pos = 0;
     self.pause_flag = 0;
 
+    self.clear_tags()
+    
     # Where to look for audio files / books
     self.base_media_dir = "/home/user/MyDocs/"
 
@@ -54,16 +87,6 @@ class GTK_Main:
     self.pl_dirs_file = "/home/user/MyDocs/.easyplayer/playlistdirs"
     self.pl_files_file = "/home/user/MyDocs/.easyplayer/playlistfiles"
 
-    self.prefs = {
-      "lastfile":0,		# id of last played file
-      "lastpos":0,		# position in seconds
-      "hideext":0,		# Hide filename extention, eg. ".mp3",".ogg"
-      "displaytimeout":1,	# Show the remaining time until timeout
-      "resumeplay":0,		# resume playing last file
-      "timeout":3,		# timeout in minutes, 3 ist 30 minutes
-      "bigbuttons":0		# Use HILDON_THUMB_SIZE if 1
-    }
-    
     # Try to load the preferences
     if os.path.isfile(self.prefs_file):
       try:
@@ -71,8 +94,18 @@ class GTK_Main:
         self.prefs = pickle.load(fd) # overwrite default values
         fd.close()
       except:
-        if (dbg): print "Can't load prefernces"
+        if (dbg): print "Can't load preferences"
     self.act_file_id = self.prefs["lastfile"]    
+
+    # Look, if the preferences are new enough
+    # If not (or some tag "version" just don't exists), overwrite with default
+    try:
+      if self.prefs["version"] < 4:
+        self.init_prefs()
+        self.save_prefs()    
+    except:
+      self.init_prefs()
+      self.save_prefs()
     
     if self.prefs["bigbuttons"] == 1: 
       self.std_size = gtk.HILDON_SIZE_AUTO|gtk.HILDON_SIZE_THUMB_HEIGHT
@@ -80,8 +113,12 @@ class GTK_Main:
       self.std_size = gtk.HILDON_SIZE_AUTO|gtk.HILDON_SIZE_FINGER_HEIGHT
         
     program = hildon.Program.get_instance()
-    gtk.set_application_name("EasyPlayer")
 
+    if self.prefs["silenthack"] == 1: 
+      gtk.set_application_name("FMRadio")
+    else:
+      gtk.set_application_name("EasyPlayer")
+                  
     self.window = hildon.StackableWindow()
     self.window.show()
     self.window.connect("destroy", gtk.main_quit, "WM destroy")
@@ -138,6 +175,11 @@ class GTK_Main:
     self.about_btn.connect("clicked",self.menu_about_cb, self.about_btn)
     menu.append(self.about_btn)
 
+    self.info_btn = hildon.GtkButton(self.std_size)
+    self.info_btn.set_label("File Tags");
+    self.info_btn.connect("clicked",self.menu_display_tags_cb, self.info_btn)
+    menu.append(self.info_btn)
+
     menu.show_all()
     self.window.set_app_menu(menu)
         
@@ -163,7 +205,7 @@ class GTK_Main:
       if i[1] > last_dir: # must we add a new parent dir?
         last_dir = i[1]
         d = self.pl_dirs[last_dir]
-        if d == self.base_media_dir: # don't add dir if it' base dir
+        if d == self.base_media_dir or self.get_parent_dir(d) == ".sounds": # don't add dir if it is base dir
           parent = None
         else: # add a new parent  
           parent = self.treestore.append(None, [ self.get_parent_dir(d) ])    
@@ -200,10 +242,6 @@ class GTK_Main:
     panarea.add(self.treeview)
     vbox.pack_start(panarea, True, True)
 			
-    #self.entry = hildon.Entry(self.std_size)
-    #self.entry.set_text("/home/user/MyDocs/mary.ogg")
-    #vbox.pack_start(self.entry, False, True)
-
     # Create the ProgressBar
     self.pbar_eb = gtk.EventBox()
     self.pbar_eb.set_events(gtk.gdk.BUTTON_PRESS_MASK)
@@ -299,6 +337,7 @@ class GTK_Main:
         try: 
           pos_int = self.player.query_position(self.time_format, None)[0]
           pos_str = self.convert_ns(pos_int)
+          
           # Do we use timeout for suspend?
           if self.timeout <> -1:
             if self.timeout > 0: 
@@ -311,10 +350,16 @@ class GTK_Main:
               self.reset_bar()
               gtk.gdk.threads_leave()
               self.player.set_state(gst.STATE_NULL)       
-              break; 	# leave threat 
+              break; 	# leave threat
+              
+          if self.prefs["metatags"] == 1: # show metatags instead of filename
+            if self.tags["title"] <> "":
+              self.play_name = self.tags["artist"] + " - " + self.tags["title"]                          
+               
           if play_thread_id == self.play_thread_id:
             gtk.gdk.threads_enter()
-            # don't display time to suspend if we don't use timeour ot we disabled it
+            # don't display time to suspend if we don't use timeout
+            # or we disabled displaying it
             if (self.timeout == -1) or (self.prefs["displaytimeout"] == 0):
               self.pbar.set_text(self.play_name + " - " + pos_str + " / " + dur_str)
             else:
@@ -326,7 +371,9 @@ class GTK_Main:
           time.sleep(1)
         except:
           time.sleep(0.2)
-      
+      else:
+        time.sleep(1) # Sleep for one second in pause mode. 
+
   # Extracs the parent directory name of a given path
   #
   def get_parent_dir(self, path):
@@ -338,25 +385,24 @@ class GTK_Main:
   # Recursive rescan given dir and subdirs for audio files.
   # 
   def rescan(self, dir):
-    if (dbg): print "Scanning dir ",dir
+    if (dbg): print "Scanning dir ",dir,self.dc
     if not dir.endswith(os.sep):
       dir += os.sep
     self.pl_dirs.append(dir)
-    self.dc += 1
+    idx = self.dc
     d = os.listdir(dir)
     for f in d:
       if os.path.isdir(dir+f) == 1: # f is a directory
+        self.dc += 1 
         self.rescan(dir+f)
-      elif os.path.isfile(dir+f) == 1: #f is a file
-        if f.endswith(".mp3") == 1 or \
-          f.endswith(".wav") == 1 or \
-          f.endswith(".aac") == 1 or \
-          f.endswith(".ogg") == 1 or \
-          f.endswith(".MP3") == 1 or \
-          f.endswith(".WAV") == 1 or \
-          f.endswith(".AAC") == 1 or \
-          f.endswith(".OGG") == 1:
-            self.pl_files.append((f,self.dc-1))
+      elif os.path.isfile(dir+f) == 1: # f is a file
+        ls = string.lower(f)
+        if ls.endswith(".mp3") == 1 or \
+          ls.endswith(".wav") == 1 or \
+          ls.endswith(".aac") == 1 or \
+          ls.endswith(".ogg") == 1 or \
+          ls.endswith(".flac") == 1:
+            self.pl_files.append((f,idx))
             self.max_files += 1
 
   # Rescan dir for music/sound files
@@ -430,6 +476,19 @@ class GTK_Main:
     #if response == gtk.RESPONSE_DELETE_EVENT:
     #  print "show_information_note: gtk.RESPONSE_DELETE_EVENT"
 
+  # Menu "About" callback - display About...
+  #
+  def menu_display_tags_cb(self, btn, lbl):
+    note = hildon.hildon_note_new_information(self.window, \
+          "Titel: "+self.tags["title"]+"\n"  \
+          "Artist: "+self.tags["artist"]+"\n" \
+          "Album: "+self.tags["album"]+"\n"  \
+          "Track: "+str(self.tags["track-number"])+"\n" \
+          "Encoder: "+self.tags["encoder"]+"\n" \
+          "Bitrate: "+str(self.tags["bitrate"]/1000)+" kbit/s")
+          # "genre","date","encoder-version","audio-codec","nominal-bitrate"
+    gtk.Dialog.run(note)
+
   # Resets the time-out timer to its starting value (eg. 5 min to 60 min)
   #
   def reset_timer(self):
@@ -487,6 +546,7 @@ class GTK_Main:
       dir_id = self.pl_files[file_id][1]
       self.play_name = self.pl_files[file_id][0]
       filepath = self.pl_dirs[dir_id]+self.play_name
+      self.clear_tags()
       #print filepath
       if os.path.isfile(filepath):
         self.set_pause_img()
@@ -545,6 +605,10 @@ class GTK_Main:
       self.stop_h()
       err, debug = message.parse_error()
       if (dbg): print "Error: %s" % err, debug
+    elif t == gst.MESSAGE_TAG:
+      for key in message.parse_tag().keys():
+        self.tags[key] = message.structure[key]
+        #print "Tags:",key,message.structure[key]
     #else:
     #  print "Msg %d" % t 
 
@@ -642,6 +706,18 @@ class GTK_Main:
     cb3.set_active(self.prefs["bigbuttons"])
     cb3.connect("toggled", self.cb_toggled_big_btn)
     vbox.pack_start(cb3, False, True)
+    
+    cb4 = hildon.CheckButton(self.std_size)
+    cb4.set_label("Output sound in silent mode")
+    cb4.set_active(self.prefs["silenthack"])
+    cb4.connect("toggled", self.cb_toggled_silent_hack)
+    vbox.pack_start(cb4, False, True)
+
+    cb5 = hildon.CheckButton(self.std_size)
+    cb5.set_label("Display Metatags")
+    cb5.set_active(self.prefs["metatags"])
+    cb5.connect("toggled", self.cb_toggled_metatags)
+    vbox.pack_start(cb5, False, True)
 
     timeouts = [ "5 min", "10 min", "20 min", "30 min", "60 min", "Never" ]
     # Create a picker button
@@ -680,6 +756,23 @@ class GTK_Main:
     self.prefs["bigbuttons"] = cb.get_active()
     self.save_prefs()
 
+  # Callback for changing beviour with silent profile
+  #
+  def cb_toggled_silent_hack(self, cb):
+    self.prefs["silenthack"] = cb.get_active()
+    self.save_prefs()
+    # Changing name once started does not work. Restart player for this to work.
+    #if cb.get_active():
+    #    gtk.set_application_name("FMRadio")
+    #else:
+    #    gtk.set_application_name("EasyPlayer")
+
+  # Callback for changing displaying of Filename or Metatags
+  #
+  def cb_toggled_metatags(self, cb):
+    self.prefs["metatags"] = cb.get_active()
+    self.save_prefs()
+
   # Callback for changing timeout/suspend 
   #
   def timeout_changed(self, picker):
@@ -687,6 +780,31 @@ class GTK_Main:
     self.reset_timer()
     self.save_prefs()
 
+  # Clear the taglist
+  # 
+  def clear_tags(self):  
+    self.tags = { 
+      "title":"", "artist":"", "album":"", "track-number":"", "genre":"", 
+      "date":"", "encoder":"", "encoder-version":"", "audio-codec":"", 
+      "nominal-bitrate":"", "bitrate":"" 
+    }
+
+  # Set default preferences
+  # 
+  def init_prefs(self):  
+    self.prefs = {
+      "version":4,		# Preferences version 0.4
+      "lastfile":0,		# id of last played file
+      "lastpos":0,		# position in seconds
+      "hideext":0,		# Hide filename extention, eg. ".mp3",".ogg"
+      "displaytimeout":1,	# Show the remaining time until timeout
+      "resumeplay":0,		# resume playing last file
+      "timeout":3,		# timeout in minutes, 3 is 30 minutes
+      "bigbuttons":0,		# Use HILDON_THUMB_SIZE if 1 (make Buttons even bigger)
+      "silenthack":0,		# Do not mute audio in silent profile if 1 (requires restart)
+      "metatags":0		# Display audio stream's metatags instead of filename	
+    }
+
   # Save preferences file
   #
   def save_prefs(self):
